/******************************************************************************
*   FILE:	TimRomBurn.c
*  
*   INPUT:	None
*   			 
*   DESCRIPTION:This program opens a PCI device driver and does:
*   		
*   		1. Burn the TIM (timing board) ROM with a ROM file (tim.rom).
*  
*   AUTHOR:	Scott Streit
*   DATE:	02/01/2000
*   
*
*   Revision History:
*   -----------------
*       Date            Who     Description
*   --------------------------------------------------------------------------
*       02/01/2000      sds     Initial
*
*       07/20/00        jmp     Change the file for to Burn a ROM in the
*                               timing board.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>
#include <stropts.h>
#include "TimRomBurn.h"
#include "DSPCommand.h"
#include "TIMBurn.h"


#define PCI_DEV		"/dev/astropci0\0"
#define DEBUG		0
#define READY		1
#define NOT_READY	0

int main()
{
	int pci_fd;
	int reply_value;
	int check;
	char tim_filename[80];
	char rom_filename[80];
	char proceed = '\n';
	char do_tim = '\n';
	char do_rom = '\n';
	time_t start_time = time(NULL);

	/* Initialize the filenames */
	strcpy(tim_filename, "");
	strcpy(rom_filename, "");
	
	/* Try to open a device driver connection. */
 	if ((pci_fd=open(PCI_DEV, O_RDWR)) == -1) {
    		perror("\n\nError: Could not open device driver!\n");
    		exit(EXIT_FAILURE);
    	}

	/* Inform user of consequences. */
	printf("\n****** WARNING  WARNING  WARNING  WARNING ******");
	printf("\nYou are about to overwrite the contents of your TIM board ROM.");
	printf("\nYou will not be able to recover the current version without");
	printf("\nthe original. A new ROM will need to be obtained if this does");
	printf("\nnot work. You must have used the \"rom\" script to generate");
	printf("\na tim.rom file.");

	printf("\n****** IMPORTANT IMPORTANT IMPORTANT IMPORTANT ******");
	printf("\nBefore you can burn the ROM, the \"Write Enabled\" jumper must");
	printf("\nbe set on the Timing Board. If not, the program will procede");
	printf("\nwithout error warning but nothing will actually be written");
	printf("\nto the ROM. The jumper is marked with the letters \"WE\" on");
	printf("\nthe board.");

	while (1) {
		printf("\nProceed (Y/N): ");
		proceed = getchar();
		if (proceed == 'Y' || proceed == 'y' || proceed == 'N' || proceed == 'n')
			break;

		/* Flush stdin to prevent multiple "Load ROM File (Y/N):" statements */
		/* from being output when multiple characters are entered as input.  */
		fflush(stdin);
	}
	
	/* Function getchar() leaves the \n behind, so flush it. */
	fflush(stdin);
	
	if (proceed == 'n' || proceed == 'N') {
		printf("\nNo harm done ... bye!\n\n");
		close(pci_fd);
		exit(EXIT_SUCCESS);
	}

	if (proceed == 'y' || proceed == 'Y') {
		while (1) {
			printf("\nPermanently Write TIM ROM File (Y/N): ");
			do_rom = getchar();
			if (do_rom == 'Y' || do_rom == 'y' || do_rom == 'N' || do_rom == 'n')
				break;
			
			/* Flush stdin to prevent multiple "Load ROM File (Y/N):" statements */
			/* from being output when multiple characters are entered as input.  */
			fflush(stdin);
		}
		
		/* Function getchar() leaves the \n behind, so flush it. */
		fflush(stdin);
		
		if (do_rom == 'y' || do_rom == 'Y') {
			while (strcmp(rom_filename,"") == 0) {
				printf("\nEnter TIM ROM filename (*.rom): ");
				gets(&rom_filename[0]);
			}
		}

		else if (do_rom == 'n' || do_rom == 'N') {
			printf("\nNo harm done ... bye!\n\n");
			close(pci_fd);
			exit(EXIT_SUCCESS);
		}

		if (do_rom == 'y' || do_rom == 'Y') {
			printf("\nWriting the ROM file... \n");
			check = load_rom_file(pci_fd, rom_filename);
	    		
	    		if (check == EXIT_SUCCESS) {
	    			printf("\n\nTIM ROM file successfully written.\n");
				printf("\n****** NOTE  NOTE  NOTE  NOTE  NOTE  NOTE ******");
				printf("\nTo complete the process, you must power off and on");
				printf("\nthe controller. This will load the new ROM contents");
				printf("\ninto the DSP.\n");
			}
		}
	}

	else {
		printf("\nSorry, not a valid response ... bye!\n\n");
		exit(EXIT_FAILURE);
	}
		
	/* Close the controller board connection. */
	close(pci_fd);

	printf("\nROM Burn Finished!\n");

	return 0;
}

